#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <float.h>

#include "reporter.h"
#include "queue.h"
#include "graph.h"
#include "privategraph.h"

#define TRUE 1
#define FALSE 0

/* replace "Anonymous Author" with "your name" */

char *getGraphAuthor(void) {
  return "Anonymous Author";
}

/* 

Create a topological sort of the vertices in a graph.  The result is
an array of indices for entries in the graph's Node array.  The
array is an int * block pointed to by G->topNum.

*/

void Topsort(Graph G) {
  Queue Q;			/* Queue for indegree == 0 nodes */
  int *indegree;		/* array of indegree values */
  struct Node *nodeP;		/* pointer to a Node object */
  struct Edge *edgeP;		/* pointer to an Edge object */
  Position pos;			/* pointer to a List entry */
  int count;			/* number of sorted Vertices */
  int idx;			/* an array index */
  int i;			/* loop control */

  /* Allocate the topNum array, an indegree array, and create a queue */

  if (G->topNum != NULL) {	/* discard previous sort, if any */
    free(G->topNum);
  }

  G->topNum = malloc(G->nodeCount*sizeof(int));
  FatalErrorMemory(G->topNum);

  indegree = malloc(G->nodeCount*sizeof(int));
  FatalErrorMemory(indegree);

  Q = CreateQueue(G->nodeCount);
  FatalErrorMemory(Q);

  /* Initialize the indegree array, and queue all the zero nodes */

  for (i=0; i<G->nodeCount; i++) {
    indegree[i] = G->node[i]->indegree;
    if (indegree[i] == 0) {
      Enqueue(Q,G->node[i]);
    }
  }

/* 

Dequeue a zero degree node, store its topological number in topNum,
reduce the indegree of each neighbor, enqueue neighbor if new degree
is zero, repeat until done.

*/

  count = 0;
  while (!IsEmpty(Q)) {
    nodeP = (struct Node *)Dequeue(Q);
    G->topNum[nodeP->index] = ++count;
    pos = GetFirstListEntry(G->adj[nodeP->index]);
    while (pos != NULL) {
      edgeP = (struct Edge *)GetListEntryElement(pos);
      idx = edgeP->bP->index;
      if (--indegree[idx] == 0) {
	Enqueue(Q,G->node[idx]);
      }
      pos = AdvanceListEntry(pos);
    }
  }

  /* if there are untouched nodes, there is a cycle. */

  if (count != G->nodeCount) {
    free(G->topNum);
    G->topNum = NULL;
  }

  /* Release temporary memory objects */
    
  DestroyQueue(Q);
  free(indegree);
}

/* 

Create a table of shortest path information for the graph.  The table
is an array of PathEntry elements, pointed to by G->uPathTable.  A
pointer to the starting node S (a struct Node object) is stored in
G->uPathStart.

*/
void UnweightedPathLength(Graph G, struct Node *S) {
}



