/*
Test harness for a set of procedures to read data defining a graph and
produce a simple set of nodes and edges.
*/

#include <stdio.h>
#include <stdlib.h>

#include "reporter.h"
#include "graph.h"
#include "plotgraph.h"

int main(int argc,char *argv[]) {

  FILE *src;			/* Graph descriptor file */
  Graph net;			/* pointer to a Graph header */
  struct Node *vP;		/* pointer to a Node object */

  /* read from stdin or the named file */

  if (argc == 1) {
    src = stdin;
  } else if (argc == 2) {
    src = fopen(argv[1],"r");
    if (src == NULL) {
      printf("main: file open error on %s.\n",argv[1]);
      return 1;
    }
  } else {  
    printf("usage: %s [<filename>]\n",argv[0]);
    return 1;
  }

  /* Read the definition file. */

  net = readGraph(src);
  FatalErrorMemory(net);
  fclose(src);

  /* Apply some algorithms to the graph */

  Topsort(net);
  vP = FindNodeByName(net,"v1");
  if (vP != NULL)
    UnweightedPathLength(net,vP);

  /* Print out a description of the graph in dot format */

  plotDigraphHeader(net);
  plotDigraphNodes(net);
  plotDigraphEdges(net);
  plotDigraphFooter(net);

  return 0;
}
