/*
Queue package using a fixed size array.  Manage pointers to elements.
*/

#include <stdlib.h>

#include "reporter.h"
#include "queue.h"

struct QueueRecord {
  int capacity;			/* max number of elements */
  int size;			/* current number of elements */
  int front;			/* subscript of first queue element */
  int rear;			/* subscript of last queue element */
  ElementType *elem;		/* pointer to array of element pointers */
};

Queue CreateQueue(int count) {
  Queue Q;

  if (count < 1) return NULL;

  Q = malloc(sizeof(struct QueueRecord));
  FatalErrorMemory(Q);

  Q->elem = malloc(count*sizeof(ElementType));
  FatalErrorMemory(Q->elem);

  Q->capacity = count;
  Q->size = 0;
  Q->front = 1;
  Q->rear = 0;
  return Q;
}

void DestroyQueue(Queue Q) {
  if (Q != NULL) {
    if (Q->elem != NULL) {
      free(Q->elem);
    }
    free(Q);
  }
}

void Enqueue(Queue Q, ElementType X) {
  Q->size++;
  FatalErrorBound(Q->size,Q->capacity);
  Q->rear = (Q->rear+1) % Q->capacity;
  Q->elem[Q->rear] = X;
}

ElementType Dequeue(Queue Q) {
  ElementType tmp;
  if (Q->size == 0) return NULL; /* return NULL if queue empty */
  tmp = Q->elem[Q->front];
  Q->size--;
  Q->front = (Q->front+1) % Q->capacity;
  return tmp;
}

ElementType FrontOfQueue(Queue Q) {
  if (Q->size == 0) return NULL; /* return NULL if queue empty */
  return Q->elem[Q->front];
}

int IsEmpty(Queue Q) {
  return Q->size == 0;
}

int IsFull(Queue Q) {
  return Q->size == Q->capacity;
}
