#ifndef _privategraph_h
#define _privategraph_h

#include "hash.h"

struct Node {
  char *name;			/* name of the node */
  int indegree;			/* number of incoming Edges */
  int index;			/* array subscript for this Node */
};

struct Edge {
  char *name;			/* name of the edge */
  char *nodeA;			/* name of node at start of edge */
  struct Node *aP;		/* Node object pointer for nodeA */
  char *nodeB;			/* name of node at end of edge */
  struct Node *bP;		/* Node object pointer for nodeB */
  double weight;		/* weight of edge, default 0.0 */
};

struct PathEntry {
  int known;			/* has the node been processed? */
  double dist;			/* distance from source node */
  struct Node *aP;		/* the start node of this edge */
};

struct GraphHeader {
  char *title;			/* graph title */
  struct Node **node;		/* array of pointers to Node objects */
  struct Edge **edge;		/* array of pointers to Edge objects */
  int nodeCount;		/* number of Nodes */
  int edgeCount;		/* number of Edges */
  HashTable hash;		/* key: Node name, entry: Node pointer */
  List *adj;			/* array of adjacency list pointers */
  int *topNum;			/* array of topological numbers */
  struct Node *uPathStart;	/* starting node for shortest paths */
  struct PathEntry *uPathTable;	/* array of shortest path descriptors */
};

#endif /* _privategraph_h */
